/*
editeurconfig - Editeur de Fichiers de Config de Jaxe

Copyright (C) 2011 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package editeurconfig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;
import javax.swing.text.BadLocationException;

/**
* Classe principale de l'Editeur de Schema Simple, est compose de 2 classes drivant de JPanel
*/
public class EditeurSchemaSimple extends JFrame {
    
    private EditeurFichierConfig appl;
    private File fichierXML;
    private boolean nouveau;
    private Document doc;
    private Element racine;
    private Element langage;
    private Element schema_simple;
    
    private int nbSousElements = 0;
    private int nbSousEnsembles = 0;
    private int nbParents = 0;
    
    private Object elementSelectionne, ensembleSelectionne;
    private ElementSS monElementSS;
    private EnsembleSS monEnsembleSS;
    private ArrayList<ElementSS> parents = new ArrayList<ElementSS>();
    private ArrayList<ElementSS> sousElements = new ArrayList<ElementSS>();
    private ArrayList<EnsembleSS> sousEnsembles = new ArrayList<EnsembleSS>();
    private ArrayList<AttributSS> attributs = new ArrayList<AttributSS>();
    private ArrayList<String> valeurs = new ArrayList<String>();
    
    private JTextField nomElement_t, nomEnsemble_t;
    private JComboBox texte_c;
    
    private JPanel panelHaut, panelListeElements, panelListeEnsembles;
    private JPanel panelBas, panelAffichageEns, panelListeSousElements, panelListeSousEnsembles, panelListeParents, panelOnglets;
    private JPanel listePanelAttribut;
    private JTabbedPane onglets;
    private JScrollPane defilementAffichageEl;
    
    private DefaultListModel modeleListeElements, modeleListeEnsembles, modeleListeSousElements, modeleListeSousEnsembles, modeleListeParents;
    private DefaultListModel modeleChoisirElement, modeleChoisirEnsemble;
    private JList listeElements, listeEnsembles, listeSousElements, listeSousEnsembles, listeParents;
    private JList listeChoisirElement, listeChoisirEnsemble;
    
    private JButton boutonPlus, boutonMoins, boutonSelectionner;
    
    private static final String[] attr_texte_mem = {"interdit", "autorise"};
    private static final String[] attr_texte = {Strings.get("label.interdit"), Strings.get("label.autorise")};
    private static final String[] attr_presence = {Strings.get("label.optionnelle"), Strings.get("label.obligatoire")};
    
    private DocumentListener docListenerElement, docListenerEnsemble;
    
    
    public EditeurSchemaSimple(final EditeurFichierConfig appl, final File fichierXML, final boolean nouveau, final Document doc, final Element racine) {
        super(Strings.get("titre.EditeurDeSchemaSimple"));
        setResizable(false);
        this.appl = appl;
        this.fichierXML = fichierXML;
        this.nouveau = nouveau;
        this.doc = doc;
        this.racine = racine;
        
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        afficher();
        
        langage = Outils.premierEnfantDeNom(racine, "LANGAGE");
        if (langage != null) {
            schema_simple = Outils.premierEnfantDeNom(langage, "SCHEMA_SIMPLE");
            if (schema_simple != null) {
                if (schema_simple.hasChildNodes())
                    DOMVersClasse();
            }
        }
        
        docListenerElement = new DocumentListener() {
            public void changedUpdate(DocumentEvent de) {
                ecouteurNomElement(de);
            }
            public void insertUpdate(DocumentEvent de) {
                ecouteurNomElement(de);
            }
            public void removeUpdate(DocumentEvent de) {
                ecouteurNomElement(de);
            }
        };
        
        docListenerEnsemble = new DocumentListener() {
            public void changedUpdate(DocumentEvent de) {
                ecouteurNomEnsemble(de);;
            }
            public void insertUpdate(DocumentEvent de) {
                ecouteurNomEnsemble(de);
            }
            public void removeUpdate(DocumentEvent de) {
                ecouteurNomEnsemble(de);
            }
        };
    }
    
    /**
    * Affiche les listes des elements, ensembles, sous-elements, sous-ensembles et parents
    */
    private void afficher() {
        creerPanelHaut();
        creerPanelBas();
        creerPanelBoutons();
        
        pack();
        final Dimension dim = getSize();
        final Dimension ecran = getToolkit().getScreenSize();
        setLocation((ecran.width - dim.width)/2, (ecran.height - dim.height)/2);
        setVisible(true);
        
        // couteur liste Elements
        listeElements.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) {
                elementSelectionne = listeElements.getSelectedValue();
                if (elementSelectionne != null) {
                    if (elementSelectionne instanceof ElementSS) {
                        monEnsembleSS = null;
                        monElementSS = (ElementSS)elementSelectionne;
                        listeEnsembles.clearSelection();
                        panelAffichageEns.setVisible(false);
                        defilementAffichageEl.setVisible(true);
                        // supprimer ecouteur sur le nom element
                        nomElement_t.getDocument().removeDocumentListener(docListenerElement);
                        nomElement_t.setText(elementSelectionne.toString());
                        nomElement_t.requestFocus();
                        // ajouter ecouteur sur le nom element
                        nomElement_t.getDocument().addDocumentListener(docListenerElement);
                        if (! "".equals(monElementSS.getTexte()))
                            texte_c.setSelectedItem(Strings.get("label."+monElementSS.getTexte()));
                        activerComposant(panelOnglets, true);
                    }
                    if (monElementSS != null) {
                        // Sous Elements
                        nbSousElements = 0;
                        sousElements = null;
                        sousElements = monElementSS.getSousElements();
                        modeleListeSousElements.clear();
                        if (sousElements != null && sousElements.size()>0) {
                            for (int i = 0; i< sousElements.size();i++)
                                modeleListeSousElements.addElement(sousElements.get(i));
                            nbSousElements = sousElements.size();
                        }
                        // Sous Ensembles
                        nbSousEnsembles = 0;
                        sousEnsembles = null;
                        sousEnsembles = monElementSS.getSousEnsembles();
                        modeleListeSousEnsembles.clear();
                        if (sousEnsembles != null && sousEnsembles.size()>0) {
                            for (int i = 0; i< sousEnsembles.size();i++)
                                modeleListeSousEnsembles.addElement(sousEnsembles.get(i));
                            nbSousEnsembles = sousEnsembles.size();
                        }
                        // parents
                        nbParents = 0;
                        parents = null;
                        parents = monElementSS.getParents();
                        modeleListeParents.clear();
                        if (parents != null && parents.size()>0) {
                            for (int i = 0; i< parents.size();i++)
                                modeleListeParents.addElement(parents.get(i));
                            nbParents = parents.size();
                        }
                        
                        reconstruireOnglets();
                        // Attributs
                        recupererAttributs();
                    }
                }
                else {
                    panelAffichageEns.setVisible(true);
                    nomEnsemble_t.setEnabled(false);
                    //nomEnsemble_t.setText("");
                    defilementAffichageEl.setVisible(false);
                    activerComposant(panelOnglets, false);
                    modeleListeSousElements.clear();
                    nbSousElements = 0;
                    nbSousEnsembles = 0;
                    nbParents = 0;
                    reconstruireOnglets();
                }
            }
        });
        
        // couteur liste Ensembles
        listeEnsembles.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) {
                ensembleSelectionne = listeEnsembles.getSelectedValue();
                if (ensembleSelectionne != null) {
                    if (ensembleSelectionne instanceof EnsembleSS) {
                        monElementSS = null;
                        monEnsembleSS = (EnsembleSS)ensembleSelectionne;
                        listeElements.clearSelection();
                        defilementAffichageEl.setVisible(false);
                        panelAffichageEns.setVisible(true);
                        nomEnsemble_t.setEnabled(true);
                        // supprimer ecouteur sur le nom element
                        nomEnsemble_t.getDocument().removeDocumentListener(docListenerEnsemble);
                        nomEnsemble_t.setText(ensembleSelectionne.toString());
                        nomEnsemble_t.requestFocus();
                        // ajouter ecouteur sur le nom element
                        nomEnsemble_t.getDocument().addDocumentListener(docListenerEnsemble);
                        activerComposant(panelOnglets, true);
                    }
                    if (monEnsembleSS != null) {
                        // Sous Elements
                        nbSousElements = 0;
                        sousElements = null;
                        sousElements = monEnsembleSS.getSousElements();
                        modeleListeSousElements.clear();
                        if (sousElements != null && sousElements.size()>0) {
                            for (int i = 0; i< sousElements.size();i++)
                                modeleListeSousElements.addElement(sousElements.get(i));
                            nbSousElements = sousElements.size();
                        }
                        // Sous Ensembles
                        nbSousEnsembles = 0;
                        sousEnsembles = null;
                        sousEnsembles = monEnsembleSS.getSousEnsembles();
                        modeleListeSousEnsembles.clear();
                        if (sousEnsembles != null && sousEnsembles.size()>0) {
                            for (int i = 0; i< sousEnsembles.size();i++)
                                modeleListeSousEnsembles.addElement(sousEnsembles.get(i));
                            nbSousEnsembles = sousEnsembles.size();
                        }
                        // parents
                        nbParents = 0;
                        //parents = null;
                        //parents = monElementSS.getParents();
                        modeleListeParents.clear();
                        /*if (parents != null && parents.size()>0) {
                            for (int i = 0; i< parents.size();i++)
                                modeleListeParents.addElement(parents.get(i));
                            nbParents = parents.size();
                        }*/
                        
                        reconstruireOnglets();
                    }
                }
                else {
                    panelAffichageEns.setVisible(true);
                    nomEnsemble_t.setEnabled(false);
                    //nomEnsemble_t.setText("");
                    defilementAffichageEl.setVisible(false);
                    activerComposant(panelOnglets, false);
                }
            }
        });
    }
    
    private void creerPanelHaut() {
        creerPanelListeElements();
        creerPanelListeEnsembles();
        panelHaut = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panelHaut.add(panelListeElements);
        panelHaut.add(panelListeEnsembles);
        add(panelHaut, BorderLayout.NORTH);
        panelHaut.setBorder(BorderFactory.createLineBorder(Color.black));
    }
    
    
    private void creerPanelListeElements() {
        panelListeElements = new JPanel(new FlowLayout(FlowLayout.LEFT));
        final JPanel panLElements = new JPanel(new BorderLayout());
        
        final JLabel elements_l = new JLabel(Strings.get("label.Elements"));
        panLElements.add(elements_l, BorderLayout.NORTH);
        
        modeleListeElements = new DefaultListModel();   
        listeElements = new JList(modeleListeElements);
        listeElements.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane defilementListeElements = new JScrollPane(listeElements);
        panLElements.add(defilementListeElements);
        defilementListeElements.setPreferredSize(new Dimension(220, 200));
        
        panelListeElements.add(panLElements);
        creerPanBoutonPlusMoins(panelListeElements, true, true, false);
        boutonPlus.setAction(new AbstractAction("+") {
            public void actionPerformed(ActionEvent e) {
                nouvelElement(true);
            }
        });
        boutonMoins.setAction(new AbstractAction("-") {
            public void actionPerformed(ActionEvent e) {
                supprimerElement();
            }
        });
        
    }
    
    private void creerPanelListeEnsembles() {
        panelListeEnsembles = new JPanel(new FlowLayout(FlowLayout.LEFT));
        final JPanel panLEnsembles = new JPanel(new BorderLayout());
        
        final JLabel ensembles_l = new JLabel(Strings.get("label.Ensembles"));
        panLEnsembles.add(ensembles_l, BorderLayout.NORTH);
        
        modeleListeEnsembles = new DefaultListModel();   
        listeEnsembles = new JList(modeleListeEnsembles);
        listeEnsembles.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane defilementListeEnsembles = new JScrollPane(listeEnsembles);
        panLEnsembles.add(defilementListeEnsembles);
        defilementListeEnsembles.setPreferredSize(new Dimension(220, 200));
        
        panelListeEnsembles.add(panLEnsembles);
        creerPanBoutonPlusMoins(panelListeEnsembles, true, true, false);
        boutonPlus.setAction(new AbstractAction("+") {
            public void actionPerformed(ActionEvent e) {
                nouvelEnsemble(true);
            }
        });
        boutonMoins.setAction(new AbstractAction("-") {
            public void actionPerformed(ActionEvent e) {
                supprimerEnsemble();
            }
        });
    }
    
    
    private void creerPanelBas() {
        panelBas = new JPanel(new FlowLayout(FlowLayout.LEFT));
        creerPanelAffichageEl();
        creerPanelAffichageEns();
        creerPanelOnglets();
        add(panelBas, BorderLayout.CENTER);
    }
    
    private void creerPanelAffichageEl() {
        final JPanel panelAffichageEl = new JPanel();
        panelAffichageEl.setLayout(new BoxLayout(panelAffichageEl, BoxLayout.Y_AXIS));
        panelAffichageEl.setBorder(new TitledBorder(""));
        defilementAffichageEl = new JScrollPane(panelAffichageEl, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        defilementAffichageEl.setPreferredSize(new Dimension(400, 319));
        panelBas.add(defilementAffichageEl);
        defilementAffichageEl.setVisible(false);
        
        // Nom Element
        final JPanel panNomElement = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panelAffichageEl.add(panNomElement);
        final JLabel nomElement_l = new JLabel(Strings.get("label.Nom"));
        nomElement_t = new JTextField(15);
        panNomElement.add(nomElement_l);
        panNomElement.add(nomElement_t);
        
        // texte
        final JPanel panTexte = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panelAffichageEl.add(panTexte);
        final JLabel texte_l = new JLabel(Strings.get("label.Texte"));
        texte_c = new JComboBox(attr_texte);
        texte_c.setSelectedIndex(0);
        panTexte.add(texte_l);
        panTexte.add(texte_c);
        // couteur sur texte
        texte_c.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent ie) {
                if (texte_c.getSelectedItem() != null) {
                    monElementSS.setTexte(attr_texte_mem[texte_c.getSelectedIndex()]);
                }
            }
        });
        
        // Attribut
        listePanelAttribut = new JPanel();
        listePanelAttribut.setLayout(new BoxLayout(listePanelAttribut, BoxLayout.Y_AXIS));
        panelAffichageEl.add(listePanelAttribut);
    }
    
    private void creerPanelAffichageEns() {
        panelAffichageEns = new JPanel();
        panelAffichageEns.setBorder(new TitledBorder(""));
        panelAffichageEns.setPreferredSize(new Dimension(400, 319));
        panelBas.add(panelAffichageEns);
        //panelAffichageEns.setVisible(false);
        
        // Nom Ensemble
        final JPanel panNomEnsemble = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panelAffichageEns.add(panNomEnsemble);
        final JLabel nomEnsemble_l = new JLabel(Strings.get("label.Nom"));
        nomEnsemble_t = new JTextField(15);
        nomEnsemble_t.setEnabled(false);
        panNomEnsemble.add(nomEnsemble_l);
        panNomEnsemble.add(nomEnsemble_t);
    }
    
    private void creerPanelAttribut(final AttributSS monAttributSS, final int index) {
        final JPanel panAttributTout = new JPanel(new FlowLayout(FlowLayout.LEFT));
        listePanelAttribut.add(panAttributTout);
        
        final JPanel panAttribut = new JPanel(new GridLayout(3,2));
        panAttributTout.add(panAttribut);
        panAttribut.setLayout(new BoxLayout(panAttribut, BoxLayout.Y_AXIS));
        panAttribut.setBorder(new TitledBorder(Strings.get("label.Attribut")));
        
        // nom
        final JPanel panNomAttribut = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panAttribut.add(panNomAttribut);
        final JLabel nomAttribut_l = new JLabel(Strings.get("label.Nom"));
        final JTextField nomAttribut_t = new JTextField(15);
        if (monAttributSS != null)
            nomAttribut_t.setText(monAttributSS.getNom());
        panNomAttribut.add(nomAttribut_l);
        panNomAttribut.add(nomAttribut_t);
        // ecouteur sur le nom attribut
        nomAttribut_t.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent de) {
                ecouteurNomAttribut(de, monAttributSS);
            }
            public void insertUpdate(DocumentEvent de) {
                ecouteurNomAttribut(de, monAttributSS);
            }
            public void removeUpdate(DocumentEvent de) {
                ecouteurNomAttribut(de, monAttributSS);
            }
        });
        
        // presence
        final JPanel panPresence = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panAttribut.add(panPresence);
        final JLabel presence_l = new JLabel(Strings.get("label.presence"));
        final JComboBox presence_c = new JComboBox(attr_presence);
        presence_c.setSelectedIndex(0);
        if (monAttributSS != null && !("".equals(monAttributSS.getPresence())))
            presence_c.setSelectedItem(Strings.get("label."+monAttributSS.getPresence()));
        panPresence.add(presence_l);
        panPresence.add(presence_c);
        // couteur sur presence
        presence_c.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent ie) {
                if (presence_c.getSelectedItem() != null) {
                    monAttributSS.setPresence(attr_presence[presence_c.getSelectedIndex()]);
                }
            }
        });
        
        // valeur
        final JPanel listePanelValeur = new JPanel();
        listePanelValeur.setLayout(new BoxLayout(listePanelValeur, BoxLayout.Y_AXIS));
        panAttribut.add(listePanelValeur);
        if (monAttributSS != null)
            recupererValeurs(monAttributSS, panAttribut, listePanelValeur);
        /*else
            recupererValeurs(enregistrerAttribut(index, null));*/
       
        creerPanBoutonPlusMoins(panAttributTout, true, true, false);
        boutonPlus.setAction(new AbstractAction("+") {
            public void actionPerformed(ActionEvent e) {
                if (!("".equals(nomAttribut_t.getText()))) {
                    if (presence_c.getSelectedItem() != null) {
                        creerPanelAttribut(enregistrerAttribut(listePanelAttribut.getComponentCount(), ""), listePanelAttribut.getComponentCount());
                        listePanelAttribut.revalidate();
                    }
                }
            }
        });
        boutonMoins.setAction(new AbstractAction("-") {
            public void actionPerformed(ActionEvent e) {
                if (listePanelAttribut.getComponentCount() > 1) {
                    listePanelAttribut.remove(panAttributTout);
                    listePanelAttribut.revalidate();
                    AttributSS monAttributSS = trouverAttribut(index);
                    if (monAttributSS != null)
                        monElementSS.supprimerAttribut(monAttributSS);
                }
            }
        });
    }
    
    private void creerPanelValeur(final AttributSS monAttributSS, final String valeur, final int index, final JPanel panAttribut, final JPanel listePanelValeur) {
        final JPanel panValeurTout = new JPanel(new FlowLayout(FlowLayout.LEFT));
        listePanelValeur.add(panValeurTout);
        panAttribut.add(listePanelValeur);
        
        final JPanel panVal = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panValeurTout.add(panVal);
        final JLabel valeur_l = new JLabel(Strings.get("label.Valeur"));
        final JTextField valeur_t = new JTextField(15);
        valeur_t.setText(valeur);
        panVal.add(valeur_l);
        panVal.add(valeur_t);
        // ecouteur sur la valeur
        valeur_t.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent de) {
                ecouteurValeurAttribut(de, monAttributSS, listePanelValeur.getComponentZOrder(panValeurTout));
            }
            public void insertUpdate(DocumentEvent de) {
                ecouteurValeurAttribut(de, monAttributSS, listePanelValeur.getComponentZOrder(panValeurTout));
            }
            public void removeUpdate(DocumentEvent de) {
                ecouteurValeurAttribut(de, monAttributSS, listePanelValeur.getComponentZOrder(panValeurTout));
            }
        });
        
        creerPanBoutonPlusMoins(panValeurTout, true, true, false);
        boutonPlus.setAction(new AbstractAction("+") {
            public void actionPerformed(ActionEvent e) {
                if (!("".equals(valeur_t.getText()))) {
                    creerPanelValeur(monAttributSS, enregistrerValeurAttribut(monAttributSS, listePanelValeur.getComponentCount(), ""), listePanelValeur.getComponentCount(), panAttribut, listePanelValeur);
                    listePanelValeur.revalidate();
                }
            }
        });
        boutonMoins.setAction(new AbstractAction("-") {
            public void actionPerformed(ActionEvent e) {
              if (listePanelValeur.getComponentCount() > 1) {
                    listePanelValeur.remove(panValeurTout);
                    listePanelValeur.revalidate();
                    String maValeur = trouverValeur(monAttributSS, index);
                    if (maValeur != null)
                        monAttributSS.supprimerValeur(maValeur);
                }  
            }
        });
    }
    
    private void creerPanelOnglets() {
        panelOnglets = new JPanel();
        panelBas.add(panelOnglets);
        
        onglets = new JTabbedPane(SwingConstants.TOP);
        onglets.setPreferredSize(new Dimension(420, 300));
        panelOnglets.add(onglets);
        
        // Sous-Elments
        creerPanelListeSousElements();
        onglets.addTab(Strings.get("label.SousElements")+" ("+nbSousElements+")", panelListeSousElements);
        
        // Sous-Ensembles
        creerPanelListeSousEnsembles();
        onglets.addTab(Strings.get("label.SousEnsembles")+" ("+nbSousEnsembles+")", panelListeSousEnsembles);
        
        // Parents
        creerPanelListeParents();
        onglets.addTab(Strings.get("label.Parents")+" ("+nbParents+")", panelListeParents);
        
        activerComposant(panelOnglets, false);
    }
    
    // panel liste des Sous-Elments
    private void creerPanelListeSousElements() {
        panelListeSousElements = new JPanel(new FlowLayout(FlowLayout.LEFT));
        final JPanel panLSousElements = new JPanel(new BorderLayout());
        
        final JLabel sous_elements_l = new JLabel(Strings.get("label.SousElements"));
        panLSousElements.add(sous_elements_l, BorderLayout.NORTH);
        
        modeleChoisirElement = new DefaultListModel();   
        listeChoisirElement = new JList(modeleChoisirElement);
        
        modeleListeSousElements = new DefaultListModel();   
        listeSousElements = new JList(modeleListeSousElements);
        listeSousElements.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane defilementListeSousElements = new JScrollPane(listeSousElements);
        panLSousElements.add(defilementListeSousElements);
        defilementListeSousElements.setPreferredSize(new Dimension(220, 200));
        
        panelListeSousElements.add(panLSousElements);
        creerPanBoutonPlusMoins(panelListeSousElements, true, true, true);
        boutonPlus.setAction(new AbstractAction("+") {
            public void actionPerformed(ActionEvent e) {
                ajouterSousElement();
            }
        });
        boutonMoins.setAction(new AbstractAction("-") {
            public void actionPerformed(ActionEvent e) {
                supprimerSousElement();
            }
        });
        boutonSelectionner.setAction(new AbstractAction("Slectionner") {
            public void actionPerformed(ActionEvent e) {
                if (listeSousElements.getSelectedValue() instanceof ElementSS) {
                    ElementSS monSousElement = (ElementSS)listeSousElements.getSelectedValue();
                    listeElements.setSelectedValue(monSousElement, true);
                    listeSousElements.clearSelection();
                }
            }
        });
        
        listeSousElements.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    if (listeSousElements.getSelectedValue() instanceof ElementSS) {
                        ElementSS monSousElement = (ElementSS)listeSousElements.getSelectedValue();
                        listeElements.setSelectedValue(monSousElement, true);
                        listeSousElements.clearSelection();
                    }
                }
            }
        });
    }
    
    // panel liste des Sous-Ensembles
    private void creerPanelListeSousEnsembles() {
        panelListeSousEnsembles = new JPanel(new FlowLayout(FlowLayout.LEFT));
        final JPanel panLSousEnsembles = new JPanel(new BorderLayout());
        
        final JLabel sous_ensembles_l = new JLabel(Strings.get("label.SousEnsembles"));
        panLSousEnsembles.add(sous_ensembles_l, BorderLayout.NORTH);
        
        modeleChoisirEnsemble = new DefaultListModel();   
        listeChoisirEnsemble = new JList(modeleChoisirEnsemble);
        
        modeleListeSousEnsembles =new DefaultListModel();   
        listeSousEnsembles = new JList(modeleListeSousEnsembles);
        listeSousEnsembles.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane defilementListeSousEnsembles = new JScrollPane(listeSousEnsembles);
        panLSousEnsembles.add(defilementListeSousEnsembles);
        defilementListeSousEnsembles.setPreferredSize(new Dimension(220, 200));
        
        panelListeSousEnsembles.add(panLSousEnsembles);
        creerPanBoutonPlusMoins(panelListeSousEnsembles, true, true, true);
        boutonPlus.setAction(new AbstractAction("+") {
            public void actionPerformed(ActionEvent e) {
                ajouterSousEnsemble();
            }
        });
        boutonMoins.setAction(new AbstractAction("-") {
            public void actionPerformed(ActionEvent e) {
                supprimerSousEnsemble();
            }
        });
        boutonSelectionner.setAction(new AbstractAction("Slectionner") {
            public void actionPerformed(ActionEvent e) {
                if (listeSousEnsembles.getSelectedValue() instanceof EnsembleSS) {
                    EnsembleSS monSousEnsemble = (EnsembleSS)listeSousEnsembles.getSelectedValue();
                    listeEnsembles.setSelectedValue(monSousEnsemble, true);
                    listeSousEnsembles.clearSelection();
                }
            }
        });
        
        listeSousEnsembles.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    if (listeSousEnsembles.getSelectedValue() instanceof EnsembleSS) {
                        EnsembleSS monSousEnsemble = (EnsembleSS)listeSousEnsembles.getSelectedValue();
                        listeEnsembles.setSelectedValue(monSousEnsemble, true);
                        listeSousEnsembles.clearSelection();
                    }
                }
            }
        });
    }
    
    // panel liste des Parents
    private void creerPanelListeParents() {
        panelListeParents = new JPanel(new FlowLayout(FlowLayout.LEFT));
        final JPanel panLParents = new JPanel(new BorderLayout());
        
        final JLabel parents_l = new JLabel(Strings.get("label.Parents"));
        panLParents.add(parents_l, BorderLayout.NORTH);
        
        modeleListeParents =new DefaultListModel();   
        listeParents = new JList(modeleListeParents);
        listeParents.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane defilementListeParents = new JScrollPane(listeParents);
        panLParents.add(defilementListeParents);
        defilementListeParents.setPreferredSize(new Dimension(220, 200));
        
        panelListeParents.add(panLParents);
        creerPanBoutonPlusMoins(panelListeParents, false, false, true);
        boutonSelectionner.setAction(new AbstractAction("Slectionner") {
            public void actionPerformed(ActionEvent e) {
                if (listeParents.getSelectedValue() instanceof ElementSS) {
                    ElementSS parent = (ElementSS)listeParents.getSelectedValue();
                    listeElements.setSelectedValue(parent, true);
                    listeParents.clearSelection();
                }
            }
        });
        listeParents.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    if (listeParents.getSelectedValue() instanceof ElementSS) {
                        ElementSS parent = (ElementSS)listeParents.getSelectedValue();
                        listeElements.setSelectedValue(parent, true);
                        listeParents.clearSelection();
                    }
                }
            }
        });
    }
    
    private void creerPanBoutonPlusMoins(final JPanel panelDest, final boolean bplus, final boolean bmoins, final boolean bselectionne) {
        final JPanel panPlusMoins = new JPanel(new BorderLayout());
        panelDest.add(panPlusMoins);
        
        if (bplus) {
            boutonPlus = new JButton("+");
            panPlusMoins.add(boutonPlus, BorderLayout.NORTH);
        }
        
        if (bmoins) {
            boutonMoins = new JButton("-");
            panPlusMoins.add(boutonMoins, BorderLayout.CENTER);
        }
       
        if (bselectionne) {
            boutonSelectionner = new JButton("Slectionner");
            panPlusMoins.add(boutonSelectionner, BorderLayout.SOUTH);
        }
    }
    
    private void creerPanelBoutons() {
        JPanel panelBoutons = new JPanel(new FlowLayout());
        
        final JButton bTester = new JButton(new AbstractAction("Tester") {
            public void actionPerformed(ActionEvent e) {
                ClasseVersDOM();
                EditeurFichierConfig.convertToString(doc);
            }
        });
        //panelBoutons.add(bTester);
        
        if (nouveau) {  
            final JButton bAnnuler = new JButton(new AbstractAction(Strings.get("bouton.Annuler")) {
                public void actionPerformed(ActionEvent e) {
                    setVisible(false);
                    new ChoixDuSchema(appl, doc, racine);
                }
            });
            panelBoutons.add(bAnnuler);
        }
        
        final JButton bFermer = new JButton(new AbstractAction(Strings.get("bouton.Fermer")) {
            public void actionPerformed(ActionEvent e) {
                ClasseVersDOM();
                NodeList listeNoeud = schema_simple.getElementsByTagName("SOUS-ELEMENT");
                if (listeNoeud.getLength() >= 1) {
                    ListeElements listeElements = new ListeElements(null, null, schema_simple);
                    setVisible(false);
                    new FenetrePrincipale(appl, fichierXML, doc, racine, listeElements);
                }
                else
                    JOptionPane.showMessageDialog(null, Strings.get("message.PasElementOuSousElement"));
            }
        });
        panelBoutons.add(bFermer);
        
        add(panelBoutons, BorderLayout.SOUTH);
    }
    
    
    
    
    /**
    * affiche un dialogue pour crer un nouvel Element
    */
    private void nouvelElement(final boolean selecEl) {
        final JDialog jd = new JDialog(this, Strings.get("titre.NouvelElement"), true);
        jd.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        final JPanel panel = new JPanel(new FlowLayout());
        jd.add(panel);
        final JTextField texte = new JTextField(15);
        panel.add(texte);
        
        final JButton bOK = new JButton(new AbstractAction(Strings.get("bouton.OK")) {
            public void actionPerformed(ActionEvent e) {
                actionNouvelElement(texte, jd, selecEl);
            }
        });
        panel.add(bOK);
        
        final JButton bAnnuler = new JButton(new AbstractAction(Strings.get("bouton.Annuler")) {
            public void actionPerformed(ActionEvent e) {
                jd.setVisible(false);
            }
        });
        panel.add(bAnnuler);
        
        texte.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    actionNouvelElement(texte, jd, selecEl);
                }
            }
        });
        
        jd.pack();
        final Dimension dim = getSize();
        final Dimension ecran = getToolkit().getScreenSize();
        jd.setLocation((ecran.width - dim.width)/2, (ecran.height - dim.height)/2);
        jd.setVisible(true);
    }
    
    /**
    * action pour crer un nouvel lment
    */
    private void actionNouvelElement(final JTextField texte, final JDialog jd, final boolean selecEl) {
        if (!verifCaractere(texte.getText())) {
            JOptionPane.showMessageDialog(jd, Strings.get("message.CaractereInterdit"));
            texte.requestFocus();
            texte.selectAll();
        }
        else {
            boolean trouve = false;
            ElementSS nouvelElementSS = new ElementSS(texte.getText());
            for (int i=0; i<modeleListeElements.getSize(); i++) {
                if (nouvelElementSS.toString().equals(modeleListeElements.get(i).toString())) {
                    JOptionPane.showMessageDialog(jd, Strings.get("message.ElementExiste"));
                    texte.requestFocus();
                    texte.selectAll();
                    trouve = true;
                }
            }
            if (!trouve) {
                jd.setVisible(false);
                modeleListeElements.addElement(nouvelElementSS);
                modeleChoisirElement.addElement(nouvelElementSS);
                trierListe(modeleListeElements);
                trierListe(modeleChoisirElement);
                if (selecEl)
                    listeElements.setSelectedValue(nouvelElementSS, true);
                else {
                    listeChoisirElement.setSelectedValue(nouvelElementSS, true);
                    if (monElementSS != null && monEnsembleSS == null)
                        listeElements.setSelectedValue(monElementSS, true);
                    if (monEnsembleSS != null && monElementSS == null)
                        listeEnsembles.setSelectedValue(monEnsembleSS, true);
                }
                EditeurFichierConfig.setModif(true);
            }
        }
    }
    /**
    * affiche un dialogue pour crer un nouvel Ensemble
    */
    private void nouvelEnsemble(final boolean selecEl) {
        final JDialog jd = new JDialog(this, Strings.get("titre.NouvelEnsemble"), true);
        jd.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        final JPanel panel = new JPanel(new FlowLayout());
        jd.add(panel);
        final JTextField texte = new JTextField(15);
        panel.add(texte);
        
        final JButton bOK = new JButton(new AbstractAction(Strings.get("bouton.OK")) {
            public void actionPerformed(ActionEvent e) {
                actionNouvelEnsemble(texte, jd, selecEl);
                onglets.setSelectedComponent(panelListeSousEnsembles);
            }
        });
        panel.add(bOK);
        
        final JButton bAnnuler = new JButton(new AbstractAction(Strings.get("bouton.Annuler")) {
            public void actionPerformed(ActionEvent e) {
                jd.setVisible(false);
            }
        });
        panel.add(bAnnuler);
        
        texte.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    actionNouvelEnsemble(texte, jd, selecEl);
                }
            }
        });
        
        jd.pack();
        final Dimension dim = getSize();
        final Dimension ecran = getToolkit().getScreenSize();
        jd.setLocation((ecran.width - dim.width)/2, (ecran.height - dim.height)/2);
        jd.setVisible(true);
    }
    
    /**
    * action pour crer un nouvel lment
    */
    private void actionNouvelEnsemble(final JTextField texte, final JDialog jd, final boolean selecEl) {
        if (!verifCaractere(texte.getText())) {
            JOptionPane.showMessageDialog(jd, Strings.get("message.CaractereInterdit"));
            texte.requestFocus();
            texte.selectAll();
        }
        else {
            boolean trouve = false;
            EnsembleSS nouvelEnsembleSS = new EnsembleSS(texte.getText());
            for (int i=0; i<modeleListeEnsembles.getSize(); i++) {
                if (nouvelEnsembleSS.toString().equals(modeleListeEnsembles.get(i).toString())) {
                    JOptionPane.showMessageDialog(jd, Strings.get("message.EnsembleExiste"));
                    texte.requestFocus();
                    texte.selectAll();
                    trouve = true;
                }
            }
            if (!trouve) {
                jd.setVisible(false);
                modeleListeEnsembles.addElement(nouvelEnsembleSS);
                modeleChoisirEnsemble.addElement(nouvelEnsembleSS);
                trierListe(modeleListeEnsembles);
                trierListe(modeleChoisirEnsemble);
                if (selecEl)
                    listeEnsembles.setSelectedValue(nouvelEnsembleSS, true);
                else {
                    listeChoisirEnsemble.setSelectedValue(nouvelEnsembleSS, true);
                    if (monElementSS != null && monEnsembleSS == null)
                        listeElements.setSelectedValue(monElementSS, true);
                    if (monEnsembleSS != null && monElementSS == null)
                        listeEnsembles.setSelectedValue(monEnsembleSS, true);
                }
                EditeurFichierConfig.setModif(true);
            }
        }
    }
    
    
    /**
    * supprimer un lment
    */
    private void supprimerElement() {
        if (elementSelectionne != null) {
            if (JOptionPane.showConfirmDialog(this, Strings.get("message.SupprimerCetElement"), Strings.get("titre.Message"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                for (int i=0; i<modeleListeElements.getSize(); i++) {
                    if (!monElementSS.toString().equals(modeleListeElements.get(i).toString())) {
                        ElementSS autreElementSS = (ElementSS)modeleListeElements.get(i);
                        autreElementSS.supprimerSousElement(monElementSS);
                        autreElementSS.supprimerParent(monElementSS);
                    }
                }
                
                for (int i=0; i<modeleListeEnsembles.getSize(); i++) {
                    EnsembleSS unEnsembleSS = (EnsembleSS)modeleListeEnsembles.get(i);
                    unEnsembleSS.supprimerSousElement(monElementSS);
                }
                
                majElements(null, true);
                modeleListeElements.removeElement(monElementSS);
                modeleChoisirElement.removeElement(monElementSS);
                int premier = listeElements.getFirstVisibleIndex();
                if (premier != -1)
                    listeElements.setSelectedIndex(premier);
                
                EditeurFichierConfig.setModif(true);
            }
        }
    }
    
    /**
    * supprimer un ensemble
    */
    private void supprimerEnsemble() {
        if (ensembleSelectionne != null) {
            if (JOptionPane.showConfirmDialog(this, Strings.get("message.SupprimerCetEnsemble"), Strings.get("titre.Message"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                for (int i=0; i<modeleListeEnsembles.getSize(); i++) {
                    if (!monEnsembleSS.toString().equals(modeleListeEnsembles.get(i).toString())) {
                        EnsembleSS autreEnsembleSS = (EnsembleSS)modeleListeEnsembles.get(i);
                        autreEnsembleSS.supprimerSousEnsemble(monEnsembleSS);
                    }
                }
                
                for (int i=0; i<modeleListeElements.getSize(); i++) {
                    ElementSS unElementSS = (ElementSS)modeleListeElements.get(i);
                    unElementSS.supprimerSousEnsemble(monEnsembleSS);
                }
                
                modeleListeEnsembles.removeElement(monEnsembleSS);
                modeleChoisirEnsemble.removeElement(monEnsembleSS);
                
                EditeurFichierConfig.setModif(true);
            }
        }
    }
    
    /**
    * affiche un dialogue pour ajouter un sous-element parmi les elements existants, ou le crer sinon
    */
    private void ajouterSousElement() {
        final JDialog jd = new JDialog(this, Strings.get("titre.ChoisirElement"), true);
        jd.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        
        final JPanel panChoisirElement = new JPanel(new BorderLayout());
        jd.add(panChoisirElement);
        
        listeChoisirElement.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        listeChoisirElement.clearSelection();
        
        JScrollPane defilementChoisirElement = new JScrollPane(listeChoisirElement);
        panChoisirElement.add(defilementChoisirElement, BorderLayout.NORTH);
        defilementChoisirElement.setPreferredSize(new Dimension(220, 200));
        
        final JPanel panBountons = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panChoisirElement.add(panBountons, BorderLayout.SOUTH);
        
        final JButton bNouveau = new JButton(new AbstractAction(Strings.get("bouton.Nouveau")) {
            public void actionPerformed(ActionEvent e) {
                nouvelElement(false);
            }
        });
        panBountons.add(bNouveau);
        
        final JButton bOK = new JButton(new AbstractAction(Strings.get("bouton.OK")) {
            public void actionPerformed(ActionEvent e) {
                if (listeChoisirElement.getSelectedValue() instanceof ElementSS) {
                    ElementSS monSousElement = (ElementSS)listeChoisirElement.getSelectedValue();
                    if (monElementSS != null && monEnsembleSS == null) {
                        if (modeleListeSousElements.contains(monSousElement))
                            JOptionPane.showMessageDialog(jd, Strings.get("message.SousElementExiste"));
                        else {
                            monElementSS.ajouterSousElement(monSousElement);
                            monSousElement.ajouterParent(monElementSS);
                            jd.setVisible(false);
                        }
                        sousElements = null;
                        sousElements = monElementSS.getSousElements();
                    }
                    if (monEnsembleSS != null && monElementSS == null) {
                        if (modeleListeSousElements.contains(monSousElement))
                            JOptionPane.showMessageDialog(jd, Strings.get("message.SousElementExiste"));
                        else {
                            monEnsembleSS.ajouterSousElement(monSousElement);
                            jd.setVisible(false);
                        }
                        sousElements = null;
                        sousElements = monEnsembleSS.getSousElements();
                    }
                    if (sousElements != null && sousElements.size()>0) {
                        modeleListeSousElements.clear();
                        for (int i = 0; i< sousElements.size();i++)
                            modeleListeSousElements.addElement(sousElements.get(i));
                        nbSousElements = sousElements.size();
                        reconstruireOnglets();
                    }
                    EditeurFichierConfig.setModif(true);
                }
            }
        });
        panBountons.add(bOK);
        bOK.setEnabled(false);
        
        listeChoisirElement.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) { 
                if (listeChoisirElement.getSelectedValue() != null)
                    bOK.setEnabled(true);
                else
                    bOK.setEnabled(false);
            }
        });
        
        final JButton bAnnuler = new JButton(new AbstractAction(Strings.get("bouton.Annuler")) {
            public void actionPerformed(ActionEvent e) {
                jd.setVisible(false);
            }
        });
        panBountons.add(bAnnuler);
        
        jd.pack();
        final Dimension dim = getSize();
        final Dimension ecran = getToolkit().getScreenSize();
        jd.setLocation((ecran.width - dim.width)/2, (ecran.height - dim.height)/2);
        jd.setVisible(true);
    }
    
    /**
    * affiche un dialogue pour ajouter un sous-ensemble parmi les ensembles existants, ou le crer sinon
    */
    private void ajouterSousEnsemble() {
        final JDialog jd = new JDialog(this, Strings.get("titre.ChoisirEnsemble"), true);
        jd.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        
        final JPanel panChoisirEnsemble = new JPanel(new BorderLayout());
        jd.add(panChoisirEnsemble);
        
        listeChoisirEnsemble.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        listeChoisirEnsemble.clearSelection();
        
        JScrollPane defilementChoisirEnsemble = new JScrollPane(listeChoisirEnsemble);
        panChoisirEnsemble.add(defilementChoisirEnsemble, BorderLayout.CENTER);
        defilementChoisirEnsemble.setPreferredSize(new Dimension(220, 200));
        
        final JPanel panBountons = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panChoisirEnsemble.add(panBountons, BorderLayout.SOUTH);
        
        final JButton bNouveau = new JButton(new AbstractAction(Strings.get("bouton.Nouveau")) {
            public void actionPerformed(ActionEvent e) {
                nouvelEnsemble(false);
            }
        });
        panBountons.add(bNouveau);
        
        final JButton bOK = new JButton(new AbstractAction(Strings.get("bouton.OK")) {
            public void actionPerformed(ActionEvent e) {
                if (listeChoisirEnsemble.getSelectedValue() instanceof EnsembleSS) {
                    EnsembleSS monSousEnsemble = (EnsembleSS)listeChoisirEnsemble.getSelectedValue();
                    if (monElementSS != null && monEnsembleSS == null) {
                        if (modeleListeSousEnsembles.contains(monSousEnsemble))
                            JOptionPane.showMessageDialog(jd, Strings.get("message.SousEnsembleExiste"));
                        else {
                            monElementSS.ajouterSousEnsemble(monSousEnsemble);
                            jd.setVisible(false);
                        }
                        sousEnsembles = null;
                        sousEnsembles = monElementSS.getSousEnsembles();
                    }
                    if (monEnsembleSS != null && monElementSS == null) {
                       if (modeleListeSousEnsembles.contains(monSousEnsemble))
                            JOptionPane.showMessageDialog(jd, Strings.get("message.SousEnsembleExiste"));
                        else {
                            monEnsembleSS.ajouterSousEnsemble(monSousEnsemble);
                            //monSousEnsemble.ajouterParent(monEnsembleSS);
                            jd.setVisible(false);
                        }
                        sousEnsembles = null;
                        sousEnsembles = monEnsembleSS.getSousEnsembles();
                    }
                    if (sousEnsembles != null && sousEnsembles.size()>0) {
                        modeleListeSousEnsembles.clear();
                        for (int i = 0; i< sousEnsembles.size();i++)
                            modeleListeSousEnsembles.addElement(sousEnsembles.get(i));
                        nbSousEnsembles = sousEnsembles.size();
                        reconstruireOnglets();
                        onglets.setSelectedComponent(panelListeSousEnsembles);
                    }
                    EditeurFichierConfig.setModif(true);
                }
            }
        });
        panBountons.add(bOK);
        bOK.setEnabled(false);
        
        listeChoisirEnsemble.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) { 
                if (listeChoisirEnsemble.getSelectedValue() != null)
                    bOK.setEnabled(true);
                else
                    bOK.setEnabled(false);
            }
        });
        
        final JButton bAnnuler = new JButton(new AbstractAction(Strings.get("bouton.Annuler")) {
            public void actionPerformed(ActionEvent e) {
                jd.setVisible(false);
            }
        });
        panBountons.add(bAnnuler);
        
        jd.pack();
        final Dimension dim = getSize();
        final Dimension ecran = getToolkit().getScreenSize();
        jd.setLocation((ecran.width - dim.width)/2, (ecran.height - dim.height)/2);
        jd.setVisible(true);
    }
    
    /**
    * Supprime un sous-element
    */
    private void supprimerSousElement() {
        if (listeSousElements.getSelectedValue() != null && listeSousElements.getSelectedValue() instanceof ElementSS) {
            ElementSS monSousElement = (ElementSS)listeSousElements.getSelectedValue();
            if (monElementSS != null && monEnsembleSS == null) {
                monElementSS.supprimerSousElement(monSousElement);
                monSousElement.supprimerParent(monElementSS);
                sousElements = null;
                sousElements = monElementSS.getSousElements();
            }     
            if (monEnsembleSS != null && monElementSS == null) {
                monEnsembleSS.supprimerSousElement(monSousElement);
                sousElements = null;
                sousElements = monEnsembleSS.getSousElements();
            }        
            if (sousElements != null) {
                modeleListeSousElements.clear();
                for (int i = 0; i< sousElements.size();i++)
                    modeleListeSousElements.addElement(sousElements.get(i));
                nbSousElements = sousElements.size();
                reconstruireOnglets();
            }
            EditeurFichierConfig.setModif(true);
        }
    }
    
    /**
    * Supprime un sous-ensemble
    */
    private void supprimerSousEnsemble() {
        if (listeSousEnsembles.getSelectedValue() != null && listeSousEnsembles.getSelectedValue() instanceof EnsembleSS) {
            EnsembleSS monSousEnsemble = (EnsembleSS)listeSousEnsembles.getSelectedValue();
            if (monElementSS != null && monEnsembleSS == null) {
                monElementSS.supprimerSousEnsemble(monSousEnsemble);
                sousEnsembles = null;
                sousEnsembles = monElementSS.getSousEnsembles();
            }     
            if (monEnsembleSS != null && monElementSS == null) {
                monEnsembleSS.supprimerSousEnsemble(monSousEnsemble);
                //monSousEnsemble.supprimerParent(monEnsembleSS);
                sousEnsembles = null;
                sousEnsembles = monEnsembleSS.getSousEnsembles();
            }        
            if (sousEnsembles != null) {
                modeleListeSousEnsembles.clear();
                for (int i = 0; i< sousEnsembles.size();i++)
                    modeleListeSousEnsembles.addElement(sousEnsembles.get(i));
                nbSousEnsembles = sousEnsembles.size();
                reconstruireOnglets();
                onglets.setSelectedComponent(panelListeSousEnsembles);
            }
            EditeurFichierConfig.setModif(true);
        }
    }
    
    private void reconstruireOnglets() {
        onglets.removeAll();
        onglets.addTab(Strings.get("label.SousElements")+" ("+nbSousElements+")", panelListeSousElements);
        onglets.addTab(Strings.get("label.SousEnsembles")+" ("+nbSousEnsembles+")", panelListeSousEnsembles);
        onglets.addTab(Strings.get("label.Parents")+" ("+nbParents+")", panelListeParents);
    }
    
    
    /**
    * Active ou dsactive un composant graphique
    * @param Le composant JComponent
    * @param b, true pour activer, false pour dsactiver
    */
    private void activerComposant(JComponent composant, boolean b) {
        composant.setEnabled(b) ;
        for (int i = 0 ; i < composant.getComponentCount() ; i++) {
            Component c = composant.getComponent(i) ;
            if (c instanceof JComponent) {
                activerComposant((JComponent) c, b) ;
            }
        }
    }
    
    /**
    * couteur sur le champ nom element
    * @param Un DocumentEvent
    */
    private void ecouteurNomElement(final DocumentEvent de) {
        javax.swing.text.Document source = de.getDocument();
        try {
            String texte = source.getText(0, source.getLength());
            if (!verifCaractere(texte)) {
                JOptionPane.showMessageDialog(this, Strings.get("message.CaractereInterdit"));
                //source.remove(source.getLength()-1, source.getLength());
            }
            else {
                majElements(texte, false);
                monElementSS.setNom(texte);
                
                listeElements.repaint();
                listeSousElements.repaint();
                listeParents.repaint();
                
                EditeurFichierConfig.setModif(true);
            }
        }
        catch (BadLocationException bex) {
            System.out.println(bex);
        }
    }
    
    private void majElements(final String texte, final boolean supprimer) {
        // ELEMENT RACINE
		Element langage = Outils.premierEnfantDeNom(racine, "LANGAGE");
		if (langage != null) {
			for (Node n = langage.getFirstChild(); n != null; n = n.getNextSibling()) {
                if (n instanceof Element) {
                    Element el = (Element)n;
                    if ("RACINE".equals(el.getNodeName()) && el.getAttribute("element").equals(monElementSS.getNom())) {
                        if (texte != null && supprimer == false)
                            el.setAttribute("element", texte);
                        if (supprimer == true && texte == null)
                            el.getParentNode().removeChild(el);
                    }
                }
            }
		}
        
		// ELEMENT MENU_INSERTION
        Element menus = Outils.premierEnfantDeNom(racine, "MENUS");
		if (menus != null)
            domMenus(menus, texte, supprimer);
        
        // ELEMENT AFFICHAGE_ELEMENT
		Element affichage_noeuds = Outils.premierEnfantDeNom(racine, "AFFICHAGE_NOEUDS");
		if (affichage_noeuds != null) {
			for (Node n = affichage_noeuds.getFirstChild(); n != null; n = n.getNextSibling()) {
                if (n instanceof Element) {
                    Element el = (Element)n;
                    if ("AFFICHAGE_ELEMENT".equals(el.getNodeName()) && el.getAttribute("element").equals(monElementSS.getNom())) {
                        if (texte != null && supprimer == false)
                            el.setAttribute("element", texte);
                        if (supprimer == true && texte == null)
                            el.getParentNode().removeChild(el);
                    }
                }
            }
		}
        EditeurFichierConfig.setModif(true);
	}
    
    private void domMenus(final Node nodeParent, final String texte, final boolean supprimer) {
        for (Node n = nodeParent.getFirstChild(); n != null; n = n.getNextSibling()) {
            if ("MENU".equals(n.getNodeName())) {
                domMenus(n, texte, supprimer);
                for (Node nd = n.getFirstChild(); nd != null; nd = nd.getNextSibling()) {
                    if (nd instanceof Element) {
                        Element el = (Element)nd;
                        if ("MENU_INSERTION".equals(el.getNodeName()) && el.getAttribute("nom").equals(monElementSS.getNom())) {
                            if (texte != null && supprimer == false)
                                el.setAttribute("nom", texte);
                            if (supprimer == true && texte == null)
                                el.getParentNode().removeChild(el);
                        }
                    }
                }
            }
        }
    }
    
    /**
    * couteur sur le champ nom attribut
    * @param Un DocumentEvent
    */
    private void ecouteurNomAttribut(final DocumentEvent de, final AttributSS monAttributSS) {
        javax.swing.text.Document source = de.getDocument();
        try {
            String texte = source.getText(0, source.getLength());
            if (!verifCaractere(texte)) {
                JOptionPane.showMessageDialog(this, Strings.get("message.CaractereInterdit"));
                //source.remove(source.getLength()-1, source.getLength());
            }
            else {
                monAttributSS.setNom(texte);
                //enregistrerAttribut(index, texte);
            
                EditeurFichierConfig.setModif(true);
            }
        }
        catch (BadLocationException bex) {
            System.out.println(bex);
        }
    }
    
    
    /**
    * couteur sur le champ presence attribut
    * @param Un DocumentEvent
    */
    private void ecouteurValeurAttribut(final DocumentEvent de, final AttributSS monAttributSS, final int index) {
        javax.swing.text.Document source = de.getDocument();
        try {
            String texte = source.getText(0, source.getLength());
            monAttributSS.setValeur(texte, index);
            //enregistrerValeurAttribut(monAttributSS, index, texte);
            
            EditeurFichierConfig.setModif(true);
        }
        catch (BadLocationException bex) {
            System.out.println(bex);
        }
    }
    
    /**
    * couteur sur le champ nom ensemble
    * @param Un DocumentEvent
    */
    private void ecouteurNomEnsemble(final DocumentEvent de) {
        javax.swing.text.Document source = de.getDocument();
        try {
            String texte = source.getText(0, source.getLength());
            if (!verifCaractere(texte)) {
                JOptionPane.showMessageDialog(this, Strings.get("message.CaractereInterdit"));
                //source.remove(source.getLength()-1, source.getLength());
            }
            else {
                monEnsembleSS.setNom(texte);
            
                listeEnsembles.repaint();
                listeSousEnsembles.repaint();
            
                EditeurFichierConfig.setModif(true);
            }
        }
        catch (BadLocationException bex) {
            System.out.println(bex);
        }
    }
    
    
    /************************************/
    private void recupererAttributs() {
        listePanelAttribut.removeAll();
        if (monElementSS != null) {
            attributs = null;
            attributs = monElementSS.getAttributs();
            if (attributs != null && attributs.size()>0) {
                for (int i = 0; i< attributs.size();i++)
                    creerPanelAttribut(attributs.get(i), i);
            }
            else
                creerPanelAttribut(enregistrerAttribut(0, ""), 0);
        }
    }
    
    private AttributSS enregistrerAttribut(final int index, final String texte) {
        AttributSS monAttributSS = trouverAttribut(index);
        if (monAttributSS == null) {
            monAttributSS = new AttributSS();
            monElementSS.ajouterAttribut(monAttributSS);
        }
        monAttributSS.setNom(texte);
        EditeurFichierConfig.setModif(true);
        
        return monAttributSS;
    }
    
    private AttributSS trouverAttribut(int index) {
        if (index == -1)
            return(null);
        if (monElementSS == null)
            return(null);
        int ip = 0;
        attributs = null;
        attributs = monElementSS.getAttributs();
        for (int i = 0; i< attributs.size();i++) {
            if (ip == index)
                return attributs.get(i);
            ip++;
        }
        return(null);
    }
    
    /**************************************/
    private void recupererValeurs(final AttributSS monAttributSS, final JPanel panAttribut, final JPanel listePanelValeur) {
        listePanelValeur.removeAll();
        if (monAttributSS != null) {
            valeurs = null;
            valeurs = monAttributSS.getValeurs();
            if (valeurs != null && valeurs.size()>0) {
                for (int i = 0; i< valeurs.size();i++)
                    creerPanelValeur(monAttributSS, valeurs.get(i), i, panAttribut, listePanelValeur);
            }
            else
                creerPanelValeur(monAttributSS, enregistrerValeurAttribut(monAttributSS, 0, ""), 0, panAttribut, listePanelValeur);
        }
        //listePanelValeur.revalidate();
    }
    
    private String enregistrerValeurAttribut(final AttributSS monAttributSS, final int index, final String texte) {
        String maValeur = trouverValeur(monAttributSS, index);
        if (maValeur == null) {
            monAttributSS.ajouterValeur(texte);
        }
        monAttributSS.setValeur(texte, index);
        EditeurFichierConfig.setModif(true);
        
        return texte;
    }
    
    private String trouverValeur(final AttributSS monAttributSS, int index) {
        if (index == -1)
            return(null);
        if (monAttributSS == null)
            return(null);
        int ip = 0;
        valeurs = null;
        valeurs = monAttributSS.getValeurs();
        for (int i = 0; i< valeurs.size();i++) {
            if (ip == index)
                return valeurs.get(i);
            ip++;
        }
        return(null);
    }
    
    private void trierListe(final DefaultListModel maListeModele) {
        Object[] contenu = maListeModele.toArray();  
        Arrays.sort(contenu);  
        maListeModele.removeAllElements();
        for (int i = 0; i< contenu.length;i++)
            maListeModele.addElement(contenu[i]);
    }
    
    
    
    private void ClasseVersDOM() {
        langage = Outils.premierEnfantDeNom(racine, "LANGAGE");
        if (langage != null) {
            schema_simple = Outils.premierEnfantDeNom(langage, "SCHEMA_SIMPLE");
            if (schema_simple != null)
                langage.removeChild(schema_simple);
        }
        else {
            langage = doc.createElement("LANGAGE");
            racine.insertBefore(langage, Outils.premierEnfantDeNom(racine, "STRINGS"));
        }
        
        schema_simple = doc.createElement("SCHEMA_SIMPLE");
        Element element_racine = Outils.premierEnfantDeNom(langage, "RACINE");
        if (element_racine != null) 
            langage.insertBefore(schema_simple, element_racine);
        else
            langage.appendChild(schema_simple);
        
        // ELEMENT
        if (modeleListeElements != null && modeleListeElements.getSize()>0) {
            for (int i=0; i<modeleListeElements.getSize(); i++) {
                ElementSS elementss = (ElementSS)modeleListeElements.get(i);
                Element element_element = doc.createElement("ELEMENT");
                schema_simple.appendChild(element_element);
                element_element.setAttribute("nom", elementss.getNom());
                element_element.setAttribute("texte", elementss.getTexte());
                //SOUS-ELEMENT
                ArrayList<ElementSS> sousElements = elementss.getSousElements();
                if (sousElements != null && sousElements.size()>0) {
                    for (int j = 0; j< sousElements.size();j++) {
                        ElementSS souselementss = sousElements.get(j);
                        Element element_sous_element = doc.createElement("SOUS-ELEMENT");
                        element_element.appendChild(element_sous_element);
                        element_sous_element.setAttribute("element", souselementss.getNom());
                    }
                }
                //SOUS-ENSEMBLE
                ArrayList<EnsembleSS> sousEnsembles = elementss.getSousEnsembles();
                if (sousEnsembles != null && sousEnsembles.size()>0) {
                    for (int j = 0; j< sousEnsembles.size(); j++) {
                        EnsembleSS sousensembless = sousEnsembles.get(j);
                        Element element_sous_ensemble = doc.createElement("SOUS-ENSEMBLE");
                        element_element.appendChild(element_sous_ensemble);
                        element_sous_ensemble.setAttribute("ensemble", sousensembless.getNom());
                    }
                }
                //ATTRIBUT
                ArrayList<AttributSS> attributs = elementss.getAttributs();
                if (attributs != null && attributs.size()>0) {
                    for (int j = 0; j< attributs.size(); j++) {
                        AttributSS attributss = attributs.get(j);
                        if (!"".equals(attributss.getNom())) {
                            Element element_attribut = doc.createElement("ATTRIBUT");
                            element_element.appendChild(element_attribut);
                            element_attribut.setAttribute("nom", attributss.getNom());
                            element_attribut.setAttribute("presence", attributss.getPresence());
                            //VALEUR
                            ArrayList<String> valeurs = attributss.getValeurs();
                            if (valeurs != null && valeurs.size()>0) {
                                for (int k = 0; k< valeurs.size(); k++) {
                                    String valeur = valeurs.get(k);
                                    if (!"".equals(valeur)) {
                                        Element element_valeur = doc.createElement("VALEUR");
                                        element_attribut.appendChild(element_valeur);
                                        Outils.setValeurElement(doc, element_valeur, valeur);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        
        // ENSEMBLE
        if (modeleListeEnsembles != null && modeleListeEnsembles.getSize()>0) {
            for (int i=0; i<modeleListeEnsembles.getSize(); i++) {
                EnsembleSS ensembless = (EnsembleSS)modeleListeEnsembles.get(i);
                Element element_ensemble = doc.createElement("ENSEMBLE");
                schema_simple.appendChild(element_ensemble);
                element_ensemble.setAttribute("nom", ensembless.getNom());
                //SOUS-ELEMENT
                ArrayList<ElementSS> sousElements = ensembless.getSousElements();
                if (sousElements != null && sousElements.size()>0) {
                    for (int j = 0; j< sousElements.size(); j++) {
                        ElementSS souselementss = sousElements.get(j);
                        Element element_sous_element = doc.createElement("SOUS-ELEMENT");
                        element_ensemble.appendChild(element_sous_element);
                        element_sous_element.setAttribute("element", souselementss.getNom());
                    }
                }
                //SOUS-ENSEMBLE
                ArrayList<EnsembleSS> sousEnsembles = ensembless.getSousEnsembles();
                if (sousEnsembles != null && sousEnsembles.size()>0) {
                    for (int j = 0; j< sousEnsembles.size(); j++) {
                        EnsembleSS sousensembless = sousEnsembles.get(j);
                        Element element_sous_ensemble = doc.createElement("SOUS-ENSEMBLE");
                        element_ensemble.appendChild(element_sous_ensemble);
                        element_sous_ensemble.setAttribute("ensemble", sousensembless.getNom());
                    }
                }
            }
        }
    }
    
    private void DOMVersClasse() {
        modeleListeElements.clear();
        modeleChoisirElement.clear();
        modeleListeEnsembles.clear();
        modeleChoisirEnsemble.clear();
        //remplir la liste de tous les elements
        NodeList listeNoeud = schema_simple.getElementsByTagName("ELEMENT");
        for (int i=0; i<listeNoeud.getLength(); i++) {
            if (listeNoeud.item(i) instanceof Element) {
                Element element_element = (Element)listeNoeud.item(i);
                ElementSS elementSS = new ElementSS(element_element.getAttribute("nom"));
                elementSS.setTexte(element_element.getAttribute("texte"));
                modeleListeElements.addElement(elementSS);
                modeleChoisirElement.addElement(elementSS);
                trierListe(modeleListeElements);
                trierListe(modeleChoisirElement);
            }
        }
        //remplir la liste de tous les ensembles
        NodeList listeNoeud2 = schema_simple.getElementsByTagName("ENSEMBLE");
        for (int i=0; i<listeNoeud2.getLength(); i++) {
            if (listeNoeud2.item(i) instanceof Element) {
                Element element_ensemble = (Element)listeNoeud2.item(i);
                EnsembleSS ensembleSS = new EnsembleSS(element_ensemble.getAttribute("nom"));
                modeleListeEnsembles.addElement(ensembleSS);
                modeleChoisirEnsemble.addElement(ensembleSS);
                trierListe(modeleListeEnsembles);
                trierListe(modeleChoisirEnsemble);
            }
        }
        
        for (Node n = schema_simple.getFirstChild(); n != null; n = n.getNextSibling()) {
            //ELEMENT
            if ("ELEMENT".equals(n.getNodeName())) {
                ElementSS elementSS = chercherElement(((Element)n).getAttribute("nom"), modeleListeElements);
                if (elementSS != null) {
                    for (Node nd = n.getFirstChild(); nd != null; nd = nd.getNextSibling()) {
                        if ("SOUS-ELEMENT".equals(nd.getNodeName())) {
                            ElementSS sousElementSS = chercherElement(((Element)nd).getAttribute("element"), modeleListeElements);
                            elementSS.ajouterSousElement(sousElementSS);
                            sousElementSS.ajouterParent(elementSS);
                        }
                        if ("SOUS-ENSEMBLE".equals(nd.getNodeName())) {
                            EnsembleSS sousEnsembleSS = chercherEnsemble(((Element)nd).getAttribute("ensemble"), modeleListeEnsembles);
                            elementSS.ajouterSousEnsemble(sousEnsembleSS);
                        }
                        if ("ATTRIBUT".equals(nd.getNodeName())) {
                            AttributSS attributSS = new AttributSS();
                            attributSS.setNom(((Element)nd).getAttribute("nom"));
                            attributSS.setPresence(((Element)nd).getAttribute("presence"));
                            elementSS.ajouterAttribut(attributSS);
                            for (Node ndd = nd.getFirstChild(); ndd != null; ndd = ndd.getNextSibling()) {
                                if ("VALEUR".equals(ndd.getNodeName()))
                                    attributSS.ajouterValeur(Outils.getValeurElement((Element)ndd));
                            }
                        }
                    }
                }
            }
            
            //ENSEMBLE
            if ("ENSEMBLE".equals(n.getNodeName())) {
                EnsembleSS ensembleSS = chercherEnsemble(((Element)n).getAttribute("nom"), modeleListeEnsembles);
                for (Node nd = n.getFirstChild(); nd != null; nd = nd.getNextSibling()) {
                    if ("SOUS-ELEMENT".equals(nd.getNodeName())) {
                        ElementSS sousElementSS = chercherElement(((Element)nd).getAttribute("element"), modeleListeElements);
                        ensembleSS.ajouterSousElement(sousElementSS);
                    }
                    if ("SOUS-ENSEMBLE".equals(nd.getNodeName())) {
                        EnsembleSS sousEnsembleSS = chercherEnsemble(((Element)nd).getAttribute("ensemble"), modeleListeEnsembles);
                        ensembleSS.ajouterSousEnsemble(sousEnsembleSS);
                        //sousEnsembleSS.ajouterParent(ensembleSS);
                    }
                }
            }
        }
    }
    
    
    private ElementSS chercherElement(final String s, final DefaultListModel listeModele) {
        ElementSS elSS = null;
        for (int i=0; i<listeModele.getSize(); i++) {
            if (s.equals(listeModele.get(i).toString())) {
                elSS = (ElementSS)listeModele.get(i);
                break;
            }
            else
                elSS = null;
        }
        return elSS;
    }
    
    private EnsembleSS chercherEnsemble(final String s, final DefaultListModel listeModele) {
        EnsembleSS enSS = null;
        for (int i=0; i<listeModele.getSize(); i++) {
            if (s.equals(listeModele.get(i).toString())) {
                enSS = (EnsembleSS)listeModele.get(i);
                break;
            }
            else
                enSS = null;
        }
        return enSS;
    }
    
    private boolean verifCaractere(final String s) {
        String motif = "^[A-Za-z_][-_.A-Za-z0-9]*$";
        boolean b = false;
        if (!"".equals(s)) {
            if (s.matches(motif))
                b = true;
            return b;
        }
        else
            return b;
    }
}
